/*
    Copyright (C) 2002 Kjetil S. Matheussen / Notam.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/


#include <signal.h>
#include <stdio.h>

#include <stdlib.h>

#include <dirent.h>

#include <sys/types.h>
#include <unistd.h>

#include <vstserver.h>
#include <communicate.h>


static struct Communicate *input;

void finish(int sig){
  char *homedir=getenv("HOME");
  char temp[500];

  C_delete(input);  

  sprintf(temp,"rm -f %s/.vstserver/sockets/s_*",homedir);
  system(temp);

  sprintf(temp,"rm -f %s/.vstserver/locks/l_*",homedir);
  system(temp);


  exit(0);
}


void VSTSERVER_deleteLocks(void){
  char *homedir=getenv("HOME");
  char temp[500];
  sprintf(temp,"rm -f %s/.vstserver/locks/l_*",homedir);
  system(temp);
}


bool VSTSERVER_makeDirectories(void){
  DIR *dir,*dir2,*dir3;
  char *homedir=getenv("HOME");
  char temp[500];

  sprintf(temp,"%s/.vstserver",homedir);
  dir=opendir(temp);

  if(dir==NULL){
    sprintf(temp,"mkdir %s/.vstserver",homedir);
    system(temp);
  }else{
    closedir(dir);
  }

  sprintf(temp,"%s/.vstserver/sockets",homedir);
  dir=opendir(temp);

  if(dir==NULL){
    sprintf(temp,"mkdir %s/.vstserver/sockets",homedir);
    system(temp);
  }else{
    closedir(dir);
  }

  sprintf(temp,"%s/.vstserver/locks",homedir);
  dir=opendir(temp);

  if(dir==NULL){
    sprintf(temp,"mkdir %s/.vstserver/locks",homedir);
    system(temp);
  }else{
    closedir(dir);
  }

  sprintf(temp,"%s/.vstserver/cache",homedir);
  dir=opendir(temp);

  if(dir==NULL){
    sprintf(temp,"mkdir %s/.vstserver/cache",homedir);
    system(temp);
  }else{
    closedir(dir);
  }

  sprintf(temp,"%s/.vstserver/cache",homedir);
  dir=opendir(temp);
  sprintf(temp,"%s/.vstserver/locks",homedir);
  dir2=opendir(temp);
  sprintf(temp,"%s/.vstserver/sockets",homedir);
  dir3=opendir(temp);

  if(dir==NULL || dir2==NULL || dir3==NULL){
    return false;
  }

  closedir(dir);closedir(dir2);closedir(dir3);

  return true;
}



int main(int argc,char **argv){


  if(VSTSERVER_makeDirectories()==false){
    fprintf(stderr,"\n\nVSTSERVER/main: Could not create directories in your homedirectory.\n");
    fprintf(stderr,"Check diskspace and filepermissions and try again.\n\n");
    return 1;
  }

  input=C_newInput(0);

  if(input==NULL){
    fprintf(stderr,"\n\nVSTSERVER/main: Could not create main socket.\n\n");
    fprintf(stderr,"Server is allready running or there are left\n");
    fprintf(stderr,"files in .vstserver/sockets/. Just delete them manually.\n");
    fprintf(stderr,"Or, there is something seriously wrong. (hope not)\n\n");
    return 2;
  }

  signal(SIGINT,finish);

  /* Nah. This operation can _theoretically_ screw things up. But thats hopefully not very likely. */
  VSTSERVER_deleteLocks();

  printf("\n\nVSTSERVER/main: Vstserver %s started. Waiting for requests.\n",VERSIONSTRING);


  for(;;){
    struct VSTS_serverRequest sr;
    char temp[1024];

    for(;;){
      int size;
      while(C_acceptIncomingConnection(input)==false);

      if(
	 (size=C_receive(input,&sr,sizeof(struct VSTS_serverRequest)))
	 != sizeof(struct VSTS_serverRequest)
	 )
	{
	  if(size!=-1){
	    fprintf(stderr,"VSTSERVER/main: Wrong type of request to vstserver. Size: %d\n",size);
	  }else{
	    fprintf(stderr,"VSTSERVER/main: Request failed.\n");
	  }
	}else{
	  break;
	}
    }

    switch(sr.reqtype){
    case VSTP_new:
      sprintf(
	      temp,
	      "wine vstservant.so %d %d %s &",
	      sr.client_control_id,
	      sr.client_process_id,
	      sr.pluginname
	      );
      system(temp);
      break;

    default:
      fprintf(stderr,"VSTSERVER/main: Illegal request %d.\n",sr.reqtype);
      break;
    }
  }




  return 0;

}

